=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2009-11-24 19:48:59 +0000
+++ drizzled/db.cc	2009-11-24 20:50:09 +0000
@@ -480,6 +480,151 @@
   return(error);
 }
 
+
+static int rm_table_part2(Session *session, TableList *tables)
+{
+  TableList *table;
+  char path[FN_REFLEN];
+  uint32_t path_length= 0;
+  String wrong_tables;
+  int error= 0;
+  bool foreign_key_error= false;
+
+  pthread_mutex_lock(&LOCK_open); /* Part 2 of rm a table */
+
+  /*
+    If we have the table in the definition cache, we don't have to check the
+    .frm cursor to find if the table is a normal table (not view) and what
+    engine to use.
+  */
+
+  for (table= tables; table; table= table->next_local)
+  {
+    TableShare *share;
+    table->db_type= NULL;
+    if ((share= TableShare::getShare(table->db, table->table_name)))
+      table->db_type= share->db_type();
+  }
+
+  if (lock_table_names_exclusively(session, tables))
+  {
+    pthread_mutex_unlock(&LOCK_open);
+    return 1;
+  }
+
+  /* Don't give warnings for not found errors, as we already generate notes */
+  session->no_warnings_for_error= 1;
+
+  for (table= tables; table; table= table->next_local)
+  {
+    char *db=table->db;
+    plugin::StorageEngine *table_type;
+
+    error= session->drop_temporary_table(table);
+
+    switch (error) {
+    case  0:
+      // removed temporary table
+      continue;
+    case -1:
+      error= 1;
+      goto err_with_placeholders;
+    default:
+      // temporary table not found
+      error= 0;
+    }
+
+    table_type= table->db_type;
+
+    {
+      Table *locked_table;
+      abort_locked_tables(session, db, table->table_name);
+      remove_table_from_cache(session, db, table->table_name,
+                              RTFC_WAIT_OTHER_THREAD_FLAG |
+                              RTFC_CHECK_KILLED_FLAG);
+      /*
+        If the table was used in lock tables, remember it so that
+        unlock_table_names can free it
+      */
+      if ((locked_table= drop_locked_tables(session, db, table->table_name)))
+        table->table= locked_table;
+
+      if (session->killed)
+      {
+        error= -1;
+        goto err_with_placeholders;
+      }
+      /* remove .frm cursor and engine files */
+      path_length= build_table_filename(path, sizeof(path), db, table->table_name, table->internal_tmp_table);
+    }
+
+    if ((table_type == NULL
+          && (plugin::StorageEngine::getTableDefinition(*session,
+                                                        path,
+                                                        db,
+                                                        table->table_name,
+                                                        table->internal_tmp_table) != EEXIST)))
+    {
+      // Table was not found on disk and table can't be created from engine
+      push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
+                          ER_BAD_TABLE_ERROR, ER(ER_BAD_TABLE_ERROR),
+                          table->table_name);
+    }
+    else
+    {
+      error= plugin::StorageEngine::dropTable(*session, path, db,
+                                                table->table_name,
+                                                ! (true));
+      if ((error == ENOENT || error == HA_ERR_NO_SUCH_TABLE))
+      {
+	error= 0;
+        session->clear_error();
+      }
+
+      if (error == HA_ERR_ROW_IS_REFERENCED)
+      {
+        /* the table is referenced by a foreign key constraint */
+        foreign_key_error= true;
+      }
+    }
+
+    if (error == 0 || (foreign_key_error == false))
+        write_bin_log_drop_table(session, true, db, table->table_name);
+
+    if (error)
+    {
+      if (wrong_tables.length())
+        wrong_tables.append(',');
+      wrong_tables.append(String(table->table_name,system_charset_info));
+    }
+  }
+  /*
+    It's safe to unlock LOCK_open: we have an exclusive lock
+    on the table name.
+  */
+  pthread_mutex_unlock(&LOCK_open);
+  error= 0;
+  if (wrong_tables.length())
+  {
+    if (!foreign_key_error)
+      my_printf_error(ER_BAD_TABLE_ERROR, ER(ER_BAD_TABLE_ERROR), MYF(0),
+                      wrong_tables.c_ptr());
+    else
+    {
+      my_message(ER_ROW_IS_REFERENCED, ER(ER_ROW_IS_REFERENCED), MYF(0));
+    }
+    error= 1;
+  }
+
+  pthread_mutex_lock(&LOCK_open); /* final bit in rm table lock */
+err_with_placeholders:
+  unlock_table_names(tables, NULL);
+  pthread_mutex_unlock(&LOCK_open);
+  session->no_warnings_for_error= 0;
+
+  return(error);
+}
+
 /*
   Removes files with known extensions plus.
   session MUST be set when calling this function!
@@ -560,10 +705,15 @@
       }
     }
   }
-  if (session->killed ||
-      (tot_list && mysql_rm_table_part2(session, tot_list, true, false, true)))
+  if (session->killed)
     goto err;
 
+  if (tot_list)
+  {
+    if (rm_table_part2(session, tot_list))
+      goto err;
+  }
+
   my_dirend(dirp);
 
   if (dropped_tables)

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-11-24 19:48:59 +0000
+++ drizzled/sql_table.cc	2009-11-24 20:50:09 +0000
@@ -336,7 +336,7 @@
 
 
 /* Should should be refactored to go away */
-static void write_bin_log_drop_table(Session *session, bool if_exists, const char *db_name, const char *table_name)
+void write_bin_log_drop_table(Session *session, bool if_exists, const char *db_name, const char *table_name)
 {
   ReplicationServices &replication_services= ReplicationServices::singleton();
   string built_query;
@@ -368,8 +368,6 @@
     if_exists		If set, don't give an error if table doesn't exists.
 			In this case we give an warning of level 'NOTE'
     drop_temporary	Only drop temporary tables
-    dont_log_query	Don't write query to log files. This will also not
-			generate warnings if the Cursor files doesn't exists
 
   TODO:
     When logging to the binary log, we should log
@@ -388,7 +386,7 @@
 */
 
 int mysql_rm_table_part2(Session *session, TableList *tables, bool if_exists,
-                         bool drop_temporary, bool dont_log_query)
+                         bool drop_temporary)
 {
   TableList *table;
   char path[FN_REFLEN];
@@ -484,8 +482,8 @@
     else
     {
       error= plugin::StorageEngine::dropTable(*session, path, db,
-                                                table->table_name,
-                                                ! dont_log_query);
+                                              table->table_name,
+                                              true);
       if ((error == ENOENT || error == HA_ERR_NO_SUCH_TABLE) && if_exists)
       {
 	error= 0;

=== modified file 'drizzled/sql_table.h'
--- drizzled/sql_table.h	2009-11-24 19:48:59 +0000
+++ drizzled/sql_table.h	2009-11-24 20:50:09 +0000
@@ -34,7 +34,10 @@
 namespace drizzled { namespace message { class Table; } }
 
 int mysql_rm_table_part2(Session *session, TableList *tables, bool if_exists,
-                         bool drop_temporary, bool log_query= false);
+                         bool drop_temporary);
+void write_bin_log_drop_table(Session *session,
+                              bool if_exists, const char *db_name,
+                              const char *table_name);
 bool quick_rm_table(Session& session, const char *db,
                     const char *table_name, bool is_tmp);
 void close_cached_table(Session *session, Table *table);

